#include <SPI.h>
#include <EEPROM.h>
#include  <Wire.h>
//#define TEXTMODE
#define GRAPHICMODE
#define SSD1306

//  https://github.com/Sylaina/oled-display/blob/master/lcd.h lcd.c
#define withOled 1

#define OLED_MOSI   MOSI
#define OLED_CLK    SCK
#define OLED_DC     7
#define OLED_CS     8
#define OLED_MISO   9

#define PWMpinPlus 5
#define PWMpinMinus 6

float dvmSetU ;
long dvmSetRefTime=0 ;

char sBuf[16] ;
int32_t Xresult32;

#include "myLCD.h"
#include "IOlib.h"
#include "myFont.h"

#define EEPoffsetAddress 0x10
#define EEPscaleAddress  0x14
#define EEPdvmRefAddress 0x18

//--------------------------------------------------------------------------------------------

#define MCP342X_GAIN_1X  0x00
#define MCP342X_MODE_ONESHOT  0x00
#define MCP342X_MODE_CONTINUOUS 0x10
#define MCP342X_CHANNEL_1 0x00
#define MCP342X_SIZE_18BIT  0x0C
#define MCP342X_SIZE_MASK 0x0C
#define MCP342X_RDY  0x80

uint8_t configRegShdw;
#define devAddr1 0x68
uint8_t devAddr;

#include "MCP3421.h"

float vv,ii ;
float adcVal ;

float EEPoffset(){
  float f ;
  EEPROM.get(EEPoffsetAddress,f) ;
  return f ;
  }

float EEPscale(){
  float f ;
  EEPROM.get(EEPscaleAddress,f) ;
  return f ;
  }  

float EEPdvmRef(){
  float f ;
  EEPROM.get(EEPdvmRefAddress,f) ;
  return f ;
  }
  
void displayOLED(){
  lcd_clrscr();
  charMode=NORMALSIZE ;
  //lcd_gotoxy(0,40);
  cursorPosition.x=5 ;  cursorPosition.y=35 ;
  lcd_print("o="); MYdtostrf(EEPoffset()*1000,7,5,sBuf) ; lcd_print(sBuf); lcd_print(" mV") ; 

  //lcd_gotoxy(0,50);
  cursorPosition.x=5 ;  cursorPosition.y=35+8 ;
  lcd_print("s="); MYdtostrf(EEPscale(),7,5,sBuf) ; lcd_print(sBuf); lcd_print(" A/V") ; 
  
  //lcd_gotoxy(0,60);
   cursorPosition.x=5 ;  cursorPosition.y=35+16 ;
  lcd_print("u="); MYdtostrf(vv*1000,7,5,sBuf) ; lcd_print(sBuf); lcd_print(" mV") ; 
  charMode=TRIPLESIZE ;
  //charMode=DOUBLESIZE ;
  lcd_gotoxy(0,0);
  if(ii>0) {  ii+=0.005 ; } else { ii-=0.005 ;} // rround
  MYdtostrf(ii,5,2,sBuf) ; lcd_printX(sBuf); 
  lcd_goto_xpix_y(41,0); lcd_printX(" A") ; 
  lcd_display();  
  }

void displayOLED2(){
  lcd_clrscr();
  charMode=DOUBLESIZE ;
  lcd_gotoxy(0,0);
  lcd_printX("calDVM");
 
  MYdtostrf(dvmSetU,5,2,sBuf) ; 
  cursorPosition.x=5 ;  cursorPosition.y=15 ;
  lcd_printX(sBuf); 
 
  lcd_display();   
  }

void setup() {
  pinMode(OLED_DC, OUTPUT); //
  pinMode(OLED_CS, OUTPUT); //
  pinMode(OLED_MISO, INPUT); //

  digitalWrite(OLED_DC,1) ;
  digitalWrite(OLED_CS,1) ;
  
  SPI.begin();
  SPI.beginTransaction(SPISettings(32000000, MSBFIRST, SPI_MODE0));
  SPI.setClockDivider(SPI_CLOCK_DIV2) ; 
 
  Serial.begin(115200);
  while (!Serial) {}  // wait for Serial comms to become ready
  Serial.println("Hello World") ;
  
  lcd_init(LCD_DISP_ON) ;

  Wire.begin();  // join I2C bus
  TWBR = 12;  // 400 kHz (maximum)
   
  Serial.println("Starting up");
  Serial.print("sizeof(displayBuffer)=") ; Serial.print(sizeof(displayBuffer)) ; Serial.println() ;
  Serial.println("Testing device connection...");
 
  devAddr=devAddr1 ; 
  Serial.println(MCP3421testConnection() ? "MCP342X connection successful" : "MCP342X connection failed");
  MCP3421configure(  MCP342X_MODE_ONESHOT | MCP342X_SIZE_18BIT | MCP342X_GAIN_1X );  

  dvmSetU=1.0 ;
 // dvmSetRefTime=100 ;
   
  Serial.println("end setup") ;
  }

void keyCheck(){
  if(Serial.available()==0 ){ return ; }
  //Serial.println(" Serial.available>0") ;
  char c=Serial.read() ;
  Serial.print("key c=") ; Serial.println(c) ;
  if(c=='0') {
    Serial.print("set offset :") ;
    float offset1=adcVal ;
    MYdtostrf(offset1,10,5,sBuf) ; Serial.print(sBuf); Serial.println(" V ") ; 
    EEPROM.put(EEPoffsetAddress,offset1) ;
    }
  if(c=='5') {
    Serial.print("set scale :") ;
    float scale1=0.5/vv ;
    EEPROM.put(EEPscaleAddress,scale1) ;
    MYdtostrf(scale1,10,5,sBuf) ; Serial.print(sBuf); Serial.println(" A/V ") ; 
    }  
  if(c=='1') {
    Serial.print("set scale :") ;
    float scale1=1.0/vv ;
    EEPROM.put(EEPscaleAddress,scale1) ;
    MYdtostrf(scale1,10,5,sBuf) ; Serial.print(sBuf); Serial.println(" A/V ") ; 
    }  
  if(c=='d') {
    Serial.println("set default :") ;
    float offset1=0.0416 ;
    float scale1=1.0/0.032 ;
    float dvmRef1=4.8 ;
    EEPROM.put(EEPoffsetAddress,offset1) ;
    EEPROM.put(EEPscaleAddress,scale1) ;
    EEPROM.put(EEPdvmRefAddress,dvmRef1) ;
    Serial.print("offset=") ; MYdtostrf(EEPoffset(),10,5,sBuf) ; Serial.print(sBuf); Serial.println(" V ") ; 
    Serial.print("scale =") ; MYdtostrf(EEPscale(),10,5,sBuf) ; Serial.print(sBuf); Serial.println(" A/V") ; 
    }    
  if(c=='i') {
    Serial.print("offset=") ; MYdtostrf(EEPoffset(),10,5,sBuf) ; Serial.print(sBuf); Serial.println(" V ") ; 
    Serial.print("scale =") ; MYdtostrf(EEPscale(),10,5,sBuf) ; Serial.print(sBuf); Serial.println(" A/V") ; 
    //delay(5000) ;
    }

  if(c=='+'){ 
    EEPROM.put(EEPdvmRefAddress,EEPdvmRef()-0.01) ;
    Serial.print("dvmRef=") ; Serial.println(EEPdvmRef()) ;
    dvmSetRefTime=50 ; 
    }
  if(c=='-'){ 
    EEPROM.put(EEPdvmRefAddress,EEPdvmRef()+0.01) ;
    Serial.print("dvmRef=") ; Serial.println(EEPdvmRef()) ; 
    dvmSetRefTime=50 ;
    }
  if(c=='u'){ 
    Serial.print("enter float>") ; dvmSetU=Serial.parseFloat() ;  Serial.println() ; 
    Serial.print("dvmSetU=") ; Serial.println(dvmSetU) ; }
    dvmSetRefTime=50 ;
  }

void abort(){
  Serial.print("abort()...") ; while(1){} 
  }

void dvmCal(){
  if(withOled){ displayOLED2() ; }
  Serial.print("dvmCal() timeout)=") ; Serial.print(dvmSetRefTime) ; 
  Serial.print("   dvmU=") ; Serial.print(dvmSetU) ; 
  
  Serial.println(" ") ;
  PWMsetU(dvmSetU) ;
  dvmSetRefTime-- ;
  }
  
void loop() {
  keyCheck() ;
  delay(500) ;
  if(dvmSetRefTime>0){ dvmCal() ; return ; }
  
  MCP3421startConversion();  
  MCP3421getResult(&Xresult32);
  adcVal=Xresult32/((double)(1L<<(18-1)))*2.048 ;
  vv=adcVal-EEPoffset() ;
  ii=vv*EEPscale() ;
  Serial.print("MCP3421:  v="); 
  MYdtostrf(vv,10,5,sBuf) ; Serial.print(sBuf); Serial.print(" V ") ;
  Serial.print("  i= ") ; 
  MYdtostrf(ii,10,5,(char *)sBuf) ; Serial.print(sBuf); Serial.println(" A ") ; 
  if(withOled){ displayOLED() ;}
  PWMsetU(ii) ;
  } 

void PWMsetU(float u){
  // Serial.print("PWMsetU(") ; Serial.print(u) ; Serial.println(")") ;
  PWMset(u/2.0/EEPdvmRef()*255,-u/2.0/EEPdvmRef()*255) ;
  }

void PWMset(int PWMplus, int PWMminus){
  // Serial.print("PWMset(") ; Serial.print(PWMplus) ; Serial.print(",") ; Serial.print(PWMminus) ; Serial.println(")") ;
  analogWrite(PWMpinPlus,128+PWMplus) ;
  analogWrite(PWMpinMinus,128+PWMminus) ;
  }
  
